#include "snake.h"

#ifdef HAVE_GETOPT_H
#include <getopt.h>		/* getopt_long ( ! on {Open,Free}BSD ) */
#endif

/* public functions */
int parse_cmdline_opts (int, char **);
void parse_cmdline_args (int, char **);

/* private functions */
static void usage (char *);

/*
 * get command line options
 */
int
parse_cmdline_opts (int argc, char **argv)
{
#ifdef HAVE_GETOPT_LONG
  static struct option opts[] = {
    {"config", 1, 0, 'c'},
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'v'},
    {0, 0, 0, 0}
  };
  int opts_idx;
#endif
  int c;

  extern char *configfile;	/* configuration file */

  while (1)
    {

      /*
       * scan next option argument 
       */
#ifdef HAVE_GETOPT_LONG
      c = getopt_long (argc, argv, "c:hv", opts, &opts_idx);
#else
      c = getopt (argc, argv, "c:hv");
#endif

      /*
       * end of options , return index of next non-option argument
       */
      if (c == -1)
	return optind;

      switch (c)
	{

	case 0:
	  fputs ("bug in getopt?\n", stderr);
	  break;

	case 'c':
	  configfile = optarg;
	  break;

	case 'h':
	case '?':
	  usage (argv[0]);
	  break;		/* usage will exit anyway */

	case 'v':
	  printf ("snake %s\n", VERSION);
	  exit (0);

	}			/* switch (c) */

    }				/* while (1) */
}


/* parse arguments on the command line (commands) */
void
parse_cmdline_args (int argc, char **argv)
{
  while (argc--)
    parse_command (*argv++);
}


static void
usage (char *me)
{
  fprintf (stderr, "Usage: %s [OPTIONS]\n", me);
  fputs ("", stderr);
  fputs ("  -c, --config=file    use named configuration file\n", stderr);
  fputs ("  -h, --help           show this screen and exit\n", stderr);
  fputs ("  -v, --version        output version information and exit\n",
	 stderr);
  exit (1);
}
