/*
 *    snake - a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "snake.h"
#include <stdlib.h>		/* rand, malloc */
#include <string.h>		/* memset */

#undef DEBUG_MALLOC

/*
 * public functions 
 */
int rand_between (int low, int high);
void move_cell (cell * from, int dir, int steps);
void *safe_malloc (size_t size, char *reason);
void *safe_calloc (size_t nmemb, size_t size, char *reason);
void *safe_realloc (void *ptr, size_t size, char *reason);

int
rand_between (int low, int high)
{
  int ret;
  ret =
    (int) ((double) ((high + 1) - low) * (rand () / (RAND_MAX + 1.0))) + low;

  return ret;
}

/*
 * take the ``cell'' ``steps'' steps in direction ``dir''. * wrap around the
 * screen 
 */
void
move_cell (cell * from, int dir, int steps)
{
  if (dir == UP)
    from->y = (ARENA_HEIGHT + (from->y - steps)) % ARENA_HEIGHT;

  else if (dir == LEFT)
    from->x = (ARENA_WIDTH + (from->x - steps)) % ARENA_WIDTH;

  else if (dir == DOWN)
    from->y = (from->y + steps) % ARENA_HEIGHT;

  else if (dir == RIGHT)
    from->x = (from->x + steps) % ARENA_WIDTH;

}

void *
safe_malloc (size_t size, char *reason)
{
  void *ret;

#ifdef DEBUG_MALLOC
  fprintf (stderr, "safe_malloc: allocating %i bytes of memory for %s\n",
	   size, reason);
#endif

  ret = malloc (size);
  if (!ret)
    {
      perror ("malloc");
      finish ();
    }

  return ret;
}

void *
safe_calloc (size_t nmemb, size_t size, char *reason)
{
  void *ret;

#ifdef DEBUG_MALLOC
  fprintf (stderr,
	   "safe_calloc: allocating %i memory segments of %i bytes for %s\n",
	   nmemb, size, reason);
#endif

  ret = calloc (nmemb, size);
  if (!ret)
    {
      perror ("calloc");
      finish ();
    }

  return ret;
}

void *
safe_realloc (void *ptr, size_t size, char *reason)
{
  void *ret;

#ifdef DEBUG_MALLOC
  fprintf (stderr, "safe_realloc: allocating %i bytes of memory for %s\n",
	   size, reason);
#endif


  ret = realloc (ptr, size);
  if (ret != ptr)
    {
      perror ("realloc");
      finish ();
    }

  return ret;
}
