/*
 *    snake - a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "snake.h"
#include <errno.h>
#include <stdlib.h>		/* malloc */
#include <string.h>		/* strcat, ...   */
#include <stdio.h>		/* fopen, fclose, ...   */
#include <unistd.h>		/* getuid, stat, ...   */
#include <sys/types.h>		/* getpwuid, ...   */
#include <sys/stat.h>		/* stat, ...   */

#define DEBUG_CONFIG

/*
 * pubblic data 
 */
char *configfile;

/*
 * public_functions 
 */
int parse_config ();

/*
 * private functions 
 */
static char *get_config_path (void);
static FILE *open_config (void);

static char *
get_config_path ()
{
  char *confpath;

  /*
   * configfile as been specified on command line 
   */
  if (configfile)
    return configfile;

  /*
   * environmental variable SNAKE_CONF is set 
   */
  confpath = getenv ("SNAKE_CONF");
  if (confpath)
    return confpath;

  return NULL;

}


static FILE *
open_config ()
{
  char *confpath;
  FILE *conf;

  confpath = get_config_path ();
  if (!confpath)
    {
      message ("No configuration file");
      return NULL;
    }

  message ("Configuration file is %s", confpath);

  conf = fopen (confpath, "r");
  if (!conf)
    {
      message ("Can't open %s: %s", confpath, strerror (errno));
      return NULL;
    }

  return conf;
}


int
parse_config ()
{
  char buf[MAXCMDLINELEN];
  FILE *stream;

  stream = open_config ();
  if (!stream)
    {
      return 0;
    }

  while (fgets (buf, MAXCMDLINELEN, stream))
    {
      parse_command (buf);
    }

  fclose (stream);

  return 1;
}
